@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,15 +19,18 @@
 #ifndef CONTROLLER_HPP
 #define CONTROLLER_HPP
 
+// local headers
 #include "client_connection.hpp"
 #include "client_settings.hpp"
 #include "decoder/decoder.hpp"
 #include "message/message.hpp"
 #include "message/server_settings.hpp"
-#include "message/stream_tags.hpp"
-#include "metadata.hpp"
 #include "player/player.hpp"
 #include "stream.hpp"
+
+// 3rd party headers
+
+// standard headers
 #include <atomic>
 #include <thread>
 
@@ -43,7 +46,7 @@ using namespace std::chrono_literals;
 class Controller
 {
 public:
-    Controller(boost::asio::io_context& io_context, const ClientSettings& settings, std::unique_ptr<MetadataAdapter> meta);
+    Controller(boost::asio::io_context& io_context, const ClientSettings& settings); //, std::unique_ptr<MetadataAdapter> meta);
     void start();
     // void stop();
     static std::vector<std::string> getSupportedPlayerNames();
@@ -63,13 +66,12 @@ private:
     boost::asio::io_context& io_context_;
     boost::asio::steady_timer timer_;
     ClientSettings settings_;
-    std::string meta_callback_;
     SampleFormat sampleFormat_;
     std::unique_ptr<ClientConnection> clientConnection_;
     std::shared_ptr<Stream> stream_;
     std::unique_ptr<decoder::Decoder> decoder_;
     std::unique_ptr<player::Player> player_;
-    std::unique_ptr<MetadataAdapter> meta_;
+    // std::unique_ptr<MetadataAdapter> meta_;
     std::unique_ptr<msg::ServerSettings> serverSettings_;
     std::unique_ptr<msg::CodecHeader> headerChunk_;
 };
