@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
+#include "common/utils/logging.hpp"
 #include "common/utils/string_utils.hpp"
 
 using namespace std::chrono_literals;
@@ -30,6 +31,7 @@ namespace player
 
 static constexpr std::chrono::milliseconds BUFFER_TIME = 80ms;
 static constexpr int PERIODS = 4;
+static constexpr int MIN_PERIODS = 3;
 
 #define exp10(x) (exp((x)*log(10)))
 
@@ -68,15 +70,15 @@ AlsaPlayer::AlsaPlayer(boost::asio::io_context& io_context, const ClientSettings
         LOG(DEBUG, LOG_TAG) << "Mixer: " << mixer_name_ << ", device: " << mixer_device_ << "\n";
     }
 
-    buffer_time_ = BUFFER_TIME;
-    periods_ = PERIODS;
     auto params = utils::string::split_pairs(settings.parameter, ',', '=');
     if (params.find("buffer_time") != params.end())
         buffer_time_ = std::chrono::milliseconds(std::max(cpt::stoi(params["buffer_time"]), 10));
     if (params.find("fragments") != params.end())
         periods_ = std::max(cpt::stoi(params["fragments"]), 2);
 
-    LOG(INFO, LOG_TAG) << "Using buffer_time: " << buffer_time_.count() / 1000 << " ms, fragments: " << periods_ << "\n";
+    LOG(INFO, LOG_TAG) << "Using " << (buffer_time_.has_value() ? "configured" : "default")
+                       << " buffer_time: " << buffer_time_.value_or(BUFFER_TIME).count() / 1000 << " ms, " << (periods_.has_value() ? "configured" : "default")
+                       << " fragments: " << periods_.value_or(PERIODS) << "\n";
 }
 
 
@@ -193,7 +195,7 @@ void AlsaPlayer::waitForEvent()
 
         unsigned short revents;
         snd_ctl_poll_descriptors_revents(ctl_, fd_.get(), 1, &revents);
-        if (revents & POLLIN || (revents == 0))
+        if (((revents & POLLIN) != 0) || (revents == 0))
         {
             snd_ctl_event_t* event;
             snd_ctl_event_alloca(&event);
@@ -208,19 +210,17 @@ void AlsaPlayer::waitForEvent()
                     waitForEvent();
                     return;
                 }
-                // Sometimes the old volume is reported after this event has been raised.
+                // Sometimes the old volume is reported by getHardwareVolume after this event has been raised.
                 // As workaround we defer getting the volume by 20ms.
                 timer_.cancel();
                 timer_.expires_after(20ms);
                 timer_.async_wait([this](const boost::system::error_code& ec) {
                     if (!ec)
                     {
-                        double volume;
-                        bool muted;
-                        if (getHardwareVolume(volume, muted))
+                        if (getHardwareVolume(volume_, muted_))
                         {
-                            LOG(DEBUG, LOG_TAG) << "Volume: " << volume << ", muted: " << muted << "\n";
-                            notifyVolumeChange(volume, muted);
+                            LOG(DEBUG, LOG_TAG) << "Volume changed: " << volume_ << ", muted: " << muted_ << "\n";
+                            notifyVolumeChange(volume_, muted_);
                         }
                     }
                 });
@@ -261,12 +261,12 @@ void AlsaPlayer::initMixer()
         throw SnapException(std::string("Failed to open mixer, error: ") + snd_strerror(err));
     if ((err = snd_mixer_attach(mixer_, mixer_device_.c_str())) < 0)
         throw SnapException("Failed to attach mixer to " + mixer_device_ + ", error: " + snd_strerror(err));
-    if ((err = snd_mixer_selem_register(mixer_, NULL, NULL)) < 0)
+    if ((err = snd_mixer_selem_register(mixer_, nullptr, nullptr)) < 0)
         throw SnapException(std::string("Failed to register selem, error: ") + snd_strerror(err));
     if ((err = snd_mixer_load(mixer_)) < 0)
         throw SnapException(std::string("Failed to load mixer, error: ") + snd_strerror(err));
     elem_ = snd_mixer_find_selem(mixer_, sid);
-    if (!elem_)
+    if (elem_ == nullptr)
         throw SnapException("Failed to find mixer: " + mixer_name_);
 
     sd_ = boost::asio::posix::stream_descriptor(io_context_, fd_->fd);
@@ -279,7 +279,7 @@ void AlsaPlayer::initAlsa()
     std::lock_guard<std::recursive_mutex> lock(mutex_);
 
     const SampleFormat& format = stream_->getFormat();
-    unsigned int rate = format.rate();
+    uint32_t rate = format.rate();
     int channels = format.channels();
     int err;
 
@@ -298,6 +298,18 @@ void AlsaPlayer::initAlsa()
     if ((err = snd_pcm_hw_params_any(handle_, params)) < 0)
         throw SnapException("Can't fill params: " + string(snd_strerror(err)));
 
+    snd_output_t* output;
+    if (snd_output_buffer_open(&output) == 0)
+    {
+        if (snd_pcm_hw_params_dump(params, output) == 0)
+        {
+            char* str;
+            size_t len = snd_output_buffer_string(output, &str);
+            LOG(DEBUG, LOG_TAG) << std::string(str, len) << "\n";
+        }
+        snd_output_close(output);
+    }
+
     // Set parameters
     if ((err = snd_pcm_hw_params_set_access(handle_, params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
         throw SnapException("Can't set interleaved mode: " + string(snd_strerror(err)));
@@ -333,9 +345,9 @@ void AlsaPlayer::initAlsa()
     {
         stringstream ss;
         ss << "Can't set format: " << string(snd_strerror(err)) << ", supported: ";
-        for (int format = 0; format <= (int)SND_PCM_FORMAT_LAST; format++)
+        for (int format = 0; format <= static_cast<int>(SND_PCM_FORMAT_LAST); format++)
         {
-            snd_pcm_format_t snd_pcm_format = static_cast<snd_pcm_format_t>(format);
+            auto snd_pcm_format = static_cast<snd_pcm_format_t>(format);
             if (snd_pcm_hw_params_test_format(handle_, params, snd_pcm_format) == 0)
                 ss << snd_pcm_format_name(snd_pcm_format) << " ";
         }
@@ -350,25 +362,61 @@ void AlsaPlayer::initAlsa()
     if (rate != format.rate())
         LOG(WARNING, LOG_TAG) << "Could not set sample rate to " << format.rate() << " Hz, using: " << rate << " Hz\n";
 
-    unsigned int buffer_time = buffer_time_.count();
+    uint32_t period_time = buffer_time_.value_or(BUFFER_TIME).count() / periods_.value_or(PERIODS);
+    uint32_t max_period_time = period_time;
+    if ((err = snd_pcm_hw_params_get_period_time_max(params, &max_period_time, nullptr)) < 0)
+    {
+        LOG(ERROR, LOG_TAG) << "Can't get max period time: " << snd_strerror(err) << "\n";
+    }
+    else
+    {
+        if (period_time > max_period_time)
+        {
+            LOG(INFO, LOG_TAG) << "Period time too large, changing from " << period_time << " to " << max_period_time << "\n";
+            period_time = max_period_time;
+        }
+    }
+    uint32_t min_period_time = period_time;
+    if ((err = snd_pcm_hw_params_get_period_time_min(params, &min_period_time, nullptr)) < 0)
+    {
+        LOG(ERROR, LOG_TAG) << "Can't get min period time: " << snd_strerror(err) << "\n";
+    }
+    else
+    {
+        if (period_time < min_period_time)
+        {
+            LOG(INFO, LOG_TAG) << "Period time too small, changing from " << period_time << " to " << min_period_time << "\n";
+            period_time = min_period_time;
+        }
+    }
+
+    if ((err = snd_pcm_hw_params_set_period_time_near(handle_, params, &period_time, nullptr)) < 0)
+        throw SnapException("Can't set period time: " + string(snd_strerror(err)));
+
+    uint32_t buffer_time = buffer_time_.value_or(BUFFER_TIME).count();
+    uint32_t periods = periods_.value_or(MIN_PERIODS);
+    if (buffer_time < period_time * periods)
+    {
+        LOG(INFO, LOG_TAG) << "Buffer time smaller than " << periods << " * periods: " << buffer_time << " us < " << period_time * periods
+                           << " us, raising buffer time\n";
+        buffer_time = period_time * periods;
+    }
+
     if ((err = snd_pcm_hw_params_set_buffer_time_near(handle_, params, &buffer_time, nullptr)) < 0)
-        throw SnapException("Can't set buffer time: " + string(snd_strerror(err)));
+        throw SnapException("Can't set buffer time to " + cpt::to_string(buffer_time) + " us : " + string(snd_strerror(err)));
 
-    unsigned int periods = periods_;
-    if ((err = snd_pcm_hw_params_set_periods_near(handle_, params, &periods, 0)) < 0)
-        throw SnapException("Can't set periods: " + string(snd_strerror(err)));
-
-    //	long unsigned int periodsize = stream_->format.msRate() * 50;//2*rate/50;
-    //	if ((pcm = snd_pcm_hw_params_set_buffer_size_near(pcm_handle, params, &periodsize)) < 0)
-    //		LOG(ERROR, LOG_TAG) << "Unable to set buffer size " << (long int)periodsize << ": " <<  snd_strerror(pcm) << "\n";
+    // unsigned int periods = periods_;
+    // if ((err = snd_pcm_hw_params_set_periods_near(handle_, params, &periods, 0)) < 0)
+    //     throw SnapException("Can't set periods: " + string(snd_strerror(err)));
 
     // Write parameters
     if ((err = snd_pcm_hw_params(handle_, params)) < 0)
         throw SnapException("Can't set hardware parameters: " + string(snd_strerror(err)));
 
     // Resume information
-    unsigned int period_time;
-    snd_pcm_hw_params_get_period_time(params, &period_time, nullptr);
+    // uint32_t periods;
+    if (snd_pcm_hw_params_get_periods(params, &periods, nullptr) < 0)
+        periods = round(static_cast<double>(buffer_time) / static_cast<double>(period_time));
     snd_pcm_hw_params_get_period_size(params, &frames_, nullptr);
     LOG(INFO, LOG_TAG) << "PCM name: " << snd_pcm_name(handle_) << ", sample rate: " << rate << " Hz, channels: " << channels
                        << ", buffer time: " << buffer_time << " us, periods: " << periods << ", period time: " << period_time
@@ -554,14 +602,22 @@ void AlsaPlayer::worker()
             continue;
         }
 
-        if (framesAvail < static_cast<snd_pcm_sframes_t>(frames_))
+        // if (framesAvail < static_cast<snd_pcm_sframes_t>(frames_))
+        // {
+        //     this_thread::sleep_for(5ms);
+        //     continue;
+        // }
+        if (framesAvail == 0)
         {
-            this_thread::sleep_for(5ms);
+            auto frame_time = std::chrono::microseconds(static_cast<int>(frames_ / format.usRate()));
+            std::chrono::microseconds wait = std::min(frame_time / 2, std::chrono::microseconds(10ms));
+            LOG(DEBUG, LOG_TAG) << "No frames available, waiting for " << wait.count() << " us\n";
+            this_thread::sleep_for(wait);
             continue;
         }
 
         // LOG(TRACE, LOG_TAG) << "res: " << result << ", framesAvail: " << framesAvail << ", delay: " << framesDelay << ", frames: " << frames_ << "\n";
-        chronos::usec delay(static_cast<chronos::usec::rep>(1000 * (double)framesDelay / format.msRate()));
+        chronos::usec delay(static_cast<chronos::usec::rep>(1000 * static_cast<double>(framesDelay) / format.msRate()));
         // LOG(TRACE, LOG_TAG) << "delay: " << framesDelay << ", delay[ms]: " << delay.count() / 1000 << ", avail: " << framesAvail << "\n";
 
         if (buffer_.size() < static_cast<size_t>(framesAvail * format.frameSize()))
@@ -589,7 +645,8 @@ void AlsaPlayer::worker()
             LOG(INFO, LOG_TAG) << "Failed to get chunk\n";
             while (active_ && !stream_->waitForChunk(100ms))
             {
-                LOG(DEBUG, LOG_TAG) << "Waiting for chunk\n";
+                static utils::logging::TimeConditional cond(2s);
+                LOG(DEBUG, LOG_TAG) << cond << "Waiting for chunk\n";
                 if ((handle_ != nullptr) && (chronos::getTickCount() - lastChunkTick > 5000))
                 {
                     LOG(NOTICE, LOG_TAG) << "No chunk received for 5000ms. Closing ALSA.\n";
@@ -646,4 +703,4 @@ vector<PcmDevice> AlsaPlayer::pcm_list()
     return result;
 }
 
-} // namespace player
+} // namespace player
\ No newline at end of file
