@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#include <assert.h>
+#include <cassert>
 #include <iostream>
 
 #include "common/aixlog.hpp"
@@ -33,6 +33,19 @@ namespace player
 static constexpr auto LOG_TAG = "FilePlayer";
 static constexpr auto kDefaultBuffer = 50ms;
 
+static constexpr auto kDescription = "Raw PCM file output";
+
+std::vector<PcmDevice> FilePlayer::pcm_list(const std::string& parameter)
+{
+    auto params = utils::string::split_pairs(parameter, ',', '=');
+    string filename;
+    if (params.find("filename") != params.end())
+        filename = params["filename"];
+    if (filename.empty())
+        filename = "stdout";
+    return {PcmDevice{0, filename, kDescription}};
+}
+
 
 FilePlayer::FilePlayer(boost::asio::io_context& io_context, const ClientSettings::Player& settings, std::shared_ptr<Stream> stream)
     : Player(io_context, settings, stream), timer_(io_context), file_(nullptr)
@@ -85,20 +98,21 @@ void FilePlayer::requestAudio()
     if (buffer_.size() < needed)
         buffer_.resize(needed);
 
-    if (!stream_->getPlayerChunk(buffer_.data(), 10ms, numFrames))
+    if (!stream_->getPlayerChunkOrSilence(buffer_.data(), 10ms, numFrames))
     {
         // LOG(INFO, LOG_TAG) << "Failed to get chunk. Playing silence.\n";
-        memset(buffer_.data(), 0, needed);
     }
     else
     {
         adjustVolume(static_cast<char*>(buffer_.data()), numFrames);
     }
+
     if (file_)
     {
         fwrite(buffer_.data(), 1, needed, file_.get());
         fflush(file_.get());
     }
+
     loop();
 }
 
