@@ -16,16 +16,23 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#include <cassert>
-#include <iostream>
-
-#include <pulse/proplist.h>
+// prototype/interface header file
+#include "pulse_player.hpp"
 
+// local headers
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
+#include "common/utils/logging.hpp"
 #include "common/utils/string_utils.hpp"
-#include "pulse_player.hpp"
+
+// 3rd party headers
+#include <pulse/proplist.h>
+
+// standard headers
+#include <cassert>
+#include <iostream>
+
 
 using namespace std::chrono_literals;
 using namespace std;
@@ -136,7 +143,7 @@ vector<PcmDevice> PulsePlayer::pcm_list(const std::string& parameter)
 
 PulsePlayer::PulsePlayer(boost::asio::io_context& io_context, const ClientSettings::Player& settings, std::shared_ptr<Stream> stream)
     : Player(io_context, settings, stream), latency_(BUFFER_TIME), pa_ml_(nullptr), pa_ctx_(nullptr), playstream_(nullptr), proplist_(nullptr),
-      server_(boost::none)
+      server_(std::nullopt)
 {
     auto params = utils::string::split_pairs_to_container<std::vector<std::string>>(settings.parameter, ',', '=');
     if (params.find("buffer_time") != params.end())
@@ -177,15 +184,43 @@ bool PulsePlayer::needsThread() const
     return true;
 }
 
-
 void PulsePlayer::worker()
 {
-    pa_mainloop_run(pa_ml_, nullptr);
-}
+    while (active_)
+    {
+        pa_mainloop_run(pa_ml_, nullptr);
 
+        // if we are still active, wait for a chunk and attempt to reconnect
+        while (active_ && !stream_->waitForChunk(100ms))
+        {
+            static utils::logging::TimeConditional cond(2s);
+            LOG(DEBUG, LOG_TAG) << cond << "Waiting for a chunk to become available before reconnecting\n";
+        }
+
+        while (active_)
+        {
+            LOG(INFO, LOG_TAG) << "Chunk available, reconnecting to pulse\n";
+            try
+            {
+                connect();
+                break;
+            }
+            catch (const std::exception& e)
+            {
+                LOG(ERROR, LOG_TAG) << "Exception while connecting to pulse: " << e.what() << endl;
+                disconnect();
+                chronos::sleep(100);
+            }
+        }
+    }
+}
 
 void PulsePlayer::setHardwareVolume(double volume, bool muted)
 {
+    // if we're not connected to pulse, ignore the hardware volume change as the volume will be set upon reconnect
+    if (playstream_ == nullptr)
+        return;
+
     last_change_ = std::chrono::steady_clock::now();
     pa_cvolume cvolume;
     if (muted)
@@ -321,19 +356,38 @@ void PulsePlayer::writeCallback(pa_stream* stream, size_t nbytes)
     // LOG(TRACE, LOG_TAG) << "writeCallback latency " << usec << " us, frames: " << numFrames << "\n";
     if (!stream_->getPlayerChunkOrSilence(buffer_.data(), std::chrono::microseconds(usec), numFrames))
     {
+        // if we haven't got a chunk for a while, it's time to disconnect from pulse so the sound device
+        // can become idle/suspended.
+        if (chronos::getTickCount() - last_chunk_tick_ > 5000)
+        {
+            LOG(INFO, LOG_TAG) << "No chunk received for 5000ms, disconnecting from pulse.\n";
+            this->disconnect();
+            return;
+        }
         // LOG(TRACE, LOG_TAG) << "Failed to get chunk. Playing silence.\n";
     }
     else
     {
+        last_chunk_tick_ = chronos::getTickCount();
         adjustVolume(static_cast<char*>(buffer_.data()), numFrames);
     }
 
     pa_stream_write(stream, buffer_.data(), nbytes, nullptr, 0LL, PA_SEEK_RELATIVE);
 }
 
-
 void PulsePlayer::start()
 {
+    LOG(INFO, LOG_TAG) << "Start\n";
+
+    this->connect();
+    Player::start();
+}
+
+void PulsePlayer::connect()
+{
+    std::lock_guard<std::mutex> lock(mutex_);
+    LOG(INFO, LOG_TAG) << "Connecting to pulse\n";
+
     if (settings_.pcm_device.idx == -1)
         throw SnapException("Can't open " + settings_.pcm_device.name + ", error: No such device");
 
@@ -466,20 +520,28 @@ void PulsePlayer::start()
     if (result < 0)
         throw SnapException("Failed to connect PulseAudio playback stream");
 
-    Player::start();
+    // we just connected, pretend we got a chunk so we don't immediately disconnect.
+    last_chunk_tick_ = chronos::getTickCount();
 }
 
-
 void PulsePlayer::stop()
 {
     LOG(INFO, LOG_TAG) << "Stop\n";
+
+    this->disconnect();
+    Player::stop();
+}
+
+void PulsePlayer::disconnect()
+{
+    std::lock_guard<std::mutex> lock(mutex_);
+    LOG(INFO, LOG_TAG) << "Disconnecting from pulse\n";
+
     if (pa_ml_ != nullptr)
     {
         pa_mainloop_quit(pa_ml_, 0);
     }
 
-    Player::stop();
-
     if (pa_ctx_ != nullptr)
     {
         pa_context_disconnect(pa_ctx_);
