@@ -22,9 +22,9 @@
 #include "player.hpp"
 
 #include <atomic>
-#include <boost/optional.hpp>
 #include <cstdio>
 #include <memory>
+#include <optional>
 #include <pulse/pulseaudio.h>
 
 
@@ -52,6 +52,9 @@ protected:
     bool needsThread() const override;
     void worker() override;
 
+    void connect();
+    void disconnect();
+
     bool getHardwareVolume(double& volume, bool& muted) override;
     void setHardwareVolume(double volume, bool muted) override;
 
@@ -68,13 +71,15 @@ protected:
     int underflows_ = 0;
     std::atomic<int> pa_ready_;
 
+    long last_chunk_tick_;
+
     pa_buffer_attr bufattr_;
     pa_sample_spec pa_ss_;
     pa_mainloop* pa_ml_;
     pa_context* pa_ctx_;
     pa_stream* playstream_;
     pa_proplist* proplist_;
-    boost::optional<std::string> server_;
+    std::optional<std::string> server_;
     std::map<std::string, std::string> properties_;
 
     // cache of the last volume change
