@@ -45,8 +45,9 @@
 #include "common/str_compat.hpp"
 #include "common/utils.hpp"
 #include "common/version.hpp"
-#include "metadata.hpp"
 
+#include <boost/asio/ip/host_name.hpp>
+#include <boost/asio/signal_set.hpp>
 
 using namespace std;
 using namespace popl;
@@ -128,7 +129,6 @@ int main(int argc, char** argv)
     int exitcode = EXIT_SUCCESS;
     try
     {
-        string meta_script;
         ClientSettings settings;
         string pcm_device(player::DEFAULT_DEVICE);
 
@@ -173,8 +173,6 @@ int main(int argc, char** argv)
         else
             mixer_mode = op.add<Value<string>>("", "mixer", "software|script|none|?[:<options>]", "software");
 
-        auto metaStderr = op.add<Switch>("e", "mstderr", "send metadata to stderr");
-
 // daemon settings
 #ifdef HAS_DAEMON
         int processPriority(-3);
@@ -263,8 +261,6 @@ int main(int argc, char** argv)
             exit(EXIT_SUCCESS);
         }
 
-        // XXX: Only one metadata option must be set
-
         settings.logging.filter = logfilterOption->value();
         if (logfilterOption->is_set())
         {
@@ -428,9 +424,7 @@ int main(int argc, char** argv)
 
         LOG(INFO, LOG_TAG) << "Version " << version::code << (!version::rev().empty() ? (", revision " + version::rev(8)) : ("")) << "\n";
 
-        // Setup metadata handling
-        auto meta(metaStderr ? std::make_unique<MetaStderrAdapter>() : std::make_unique<MetadataAdapter>());
-        auto controller = make_shared<Controller>(io_context, settings, std::move(meta));
+        auto controller = make_shared<Controller>(io_context, settings);
         controller->start();
 
         int num_threads = 0;
