@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,10 +24,11 @@
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
+#include "common/utils/logging.hpp"
 #include "time_provider.hpp"
 #include <cmath>
+#include <cstring>
 #include <iostream>
-#include <string.h>
 
 
 using namespace std;
@@ -66,11 +67,6 @@ Stream::Stream(const SampleFormat& in_format, const SampleFormat& out_format)
 }
 
 
-Stream::~Stream()
-{
-}
-
-
 void Stream::setRealSampleRate(double sampleRate)
 {
     if (sampleRate == format_.rate())
@@ -87,14 +83,14 @@ void Stream::setRealSampleRate(double sampleRate)
 
 void Stream::setBufferLen(size_t bufferLenMs)
 {
-    bufferMs_ = cs::msec(bufferLenMs);
+    bufferMs_.store(cs::msec(bufferLenMs));
 }
 
 
 void Stream::clearChunks()
 {
     std::lock_guard<std::mutex> lock(mutex_);
-    while (chunks_.size() > 0)
+    while (!chunks_.empty())
         chunks_.pop();
     resetBuffers();
 }
@@ -104,7 +100,7 @@ void Stream::addChunk(unique_ptr<msg::PcmChunk> chunk)
 {
     // drop chunk if it's too old. Just in case, this shouldn't happen.
     auto age = std::chrono::duration_cast<cs::msec>(TimeProvider::serverNow() - chunk->start());
-    if (age > 5s + bufferMs_)
+    if (age > 5s + bufferMs_.load())
         return;
 
     auto resampled = resampler_->resample(std::move(chunk));
@@ -118,7 +114,7 @@ void Stream::addChunk(unique_ptr<msg::PcmChunk> chunk)
         while (chunks_.front_copy(front_))
         {
             age = std::chrono::duration_cast<cs::msec>(TimeProvider::serverNow() - front_->start());
-            if ((age > 5s + bufferMs_) && chunks_.try_pop(front_))
+            if ((age > 5s + bufferMs_.load()) && chunks_.try_pop(front_))
                 LOG(TRACE, LOG_TAG) << "Oldest chunk too old: " << age.count() << " ms, removing. Chunks in queue left: " << chunks_.size() << "\n";
             else
                 break;
@@ -207,6 +203,21 @@ cs::time_point_clk Stream::getNextPlayerChunk(void* outputBuffer, uint32_t frame
             memcpy(static_cast<char*>(outputBuffer) + pos * format_.frameSize(), read_buffer_.data() + (pos - n) * format_.frameSize(),
                    size * format_.frameSize());
         }
+        //    --      --
+        //   -  -    -  -
+        //  -    -  -    -
+        // -      --      -
+
+        //    --       --
+        //   -  -     -  -
+        //  -    --  -    -
+        // -       --      -
+
+        //    --       --
+        //   -  -     -  -
+        //  -    -_  -    -
+        // -       --      -
+
         else
         {
             // Read all input frames, but skip a frame per slice when writing to the output.
@@ -240,10 +251,10 @@ void Stream::resetBuffers()
 
 bool Stream::getPlayerChunk(void* outputBuffer, const cs::usec& outputBufferDacTime, uint32_t frames)
 {
-    if (outputBufferDacTime > bufferMs_)
+    if (outputBufferDacTime > bufferMs_.load())
     {
-        LOG(INFO, LOG_TAG) << "outputBufferDacTime > bufferMs: " << cs::duration<cs::msec>(outputBufferDacTime) << " > " << cs::duration<cs::msec>(bufferMs_)
-                           << "\n";
+        LOG(INFO, LOG_TAG) << "outputBufferDacTime > bufferMs: " << cs::duration<cs::msec>(outputBufferDacTime) << " > "
+                           << cs::duration<cs::msec>(bufferMs_.load()) << "\n";
         return false;
     }
 
@@ -281,7 +292,7 @@ bool Stream::getPlayerChunk(void* outputBuffer, const cs::usec& outputBufferDacT
         if (hard_sync_)
         {
             cs::nsec req_chunk_duration = cs::nsec(static_cast<cs::nsec::rep>(frames / format_.nsRate()));
-            cs::usec age = std::chrono::duration_cast<cs::usec>(TimeProvider::serverNow() - chunk_->start()) - bufferMs_ + outputBufferDacTime;
+            cs::usec age = std::chrono::duration_cast<cs::usec>(TimeProvider::serverNow() - chunk_->start()) - bufferMs_.load() + outputBufferDacTime;
             // LOG(INFO, LOG_TAG) << "age: " << age.count() / 1000 << ", buffer: " <<
             // std::chrono::duration_cast<chrono::milliseconds>(req_chunk_duration).count() << "\n";
             if (age < -req_chunk_duration)
@@ -302,7 +313,7 @@ bool Stream::getPlayerChunk(void* outputBuffer, const cs::usec& outputBufferDacT
                     chunk_ = nullptr;
                     while (chunks_.try_pop(chunk_))
                     {
-                        age = std::chrono::duration_cast<cs::usec>(TimeProvider::serverNow() - chunk_->start()) - bufferMs_ + outputBufferDacTime;
+                        age = std::chrono::duration_cast<cs::usec>(TimeProvider::serverNow() - chunk_->start()) - bufferMs_.load() + outputBufferDacTime;
                         LOG(DEBUG, LOG_TAG) << "age: " << age.count() / 1000 << ", requested chunk_duration: "
                                             << std::chrono::duration_cast<std::chrono::milliseconds>(req_chunk_duration).count()
                                             << ", duration: " << chunk_->duration<std::chrono::milliseconds>().count() << "\n";
@@ -333,7 +344,7 @@ bool Stream::getPlayerChunk(void* outputBuffer, const cs::usec& outputBufferDacT
                                             << ", age: " << std::chrono::duration_cast<cs::usec>(age).count() / 1000. << "\n";
                         getSilentPlayerChunk(outputBuffer, silent_frames);
                     }
-                    getNextPlayerChunk((char*)outputBuffer + (chunk_->format.frameSize() * silent_frames), frames - silent_frames);
+                    getNextPlayerChunk(static_cast<char*>(outputBuffer) + (chunk_->format.frameSize() * silent_frames), frames - silent_frames);
 
                     if (result)
                     {
@@ -352,15 +363,15 @@ bool Stream::getPlayerChunk(void* outputBuffer, const cs::usec& outputBufferDacT
         if (correctAfterXFrames_ != 0)
         {
             playedFrames_ += frames;
-            if (playedFrames_ >= (uint32_t)abs(correctAfterXFrames_))
+            if (playedFrames_ >= static_cast<uint32_t>(abs(correctAfterXFrames_)))
             {
                 framesCorrection = static_cast<int32_t>(playedFrames_) / correctAfterXFrames_;
                 playedFrames_ %= abs(correctAfterXFrames_);
             }
         }
 
-        cs::usec age = std::chrono::duration_cast<cs::usec>(TimeProvider::serverNow() - getNextPlayerChunk(outputBuffer, frames, framesCorrection) - bufferMs_ +
-                                                            outputBufferDacTime);
+        cs::usec age = std::chrono::duration_cast<cs::usec>(TimeProvider::serverNow() - getNextPlayerChunk(outputBuffer, frames, framesCorrection) -
+                                                            bufferMs_.load() + outputBufferDacTime);
 
         setRealSampleRate(format_.rate());
         // check if we need a hard sync
@@ -441,3 +452,16 @@ bool Stream::getPlayerChunk(void* outputBuffer, const cs::usec& outputBufferDacT
         return false;
     }
 }
+
+
+bool Stream::getPlayerChunkOrSilence(void* outputBuffer, const chronos::usec& outputBufferDacTime, uint32_t frames)
+{
+    bool result = getPlayerChunk(outputBuffer, outputBufferDacTime, frames);
+    if (!result)
+    {
+        static utils::logging::TimeConditional cond(1s);
+        LOG(DEBUG, LOG_TAG) << cond << "Failed to get chunk, returning silence\n";
+        getSilentPlayerChunk(outputBuffer, frames);
+    }
+    return result;
+}
