@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2021  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,15 +16,23 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef FILE_UTILS_H
-#define FILE_UTILS_H
+#ifndef FILE_UTILS_HPP
+#define FILE_UTILS_HPP
 
+// local headers
 #include "string_utils.hpp"
-#include <fstream>
+
+// 3rd party headers
+
+// standard headers
 #ifndef WINDOWS
 #include <grp.h>
 #include <pwd.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #endif
+
+#include <fstream>
 #include <stdexcept>
 #include <vector>
 
@@ -54,7 +62,7 @@ static void do_chown(const std::string& file_path, const std::string& user_name,
     if (user_name.empty() && group_name.empty())
         return;
 
-    if (!exists(file_path))
+    if (!utils::file::exists(file_path))
         return;
 
     uid_t uid = -1;
