@@ -20,6 +20,7 @@
 #define CONFIG_HPP
 
 #include <memory>
+#include <mutex>
 #include <string>
 #include <sys/time.h>
 #include <vector>
@@ -274,7 +275,7 @@ struct Group
         clients.clear();
         if (j.count("clients"))
         {
-            for (auto& jClient : j["clients"])
+            for (const auto& jClient : j["clients"])
             {
                 ClientInfoPtr client = std::make_shared<ClientInfo>();
                 client->fromJson(jClient);
@@ -293,7 +294,7 @@ struct Group
         j["muted"] = muted;
 
         json jClients = json::array();
-        for (auto client : clients)
+        for (const auto& client : clients)
             jClients.push_back(client->toJson());
         j["clients"] = jClients;
         return j;
@@ -322,7 +323,7 @@ struct Group
 
     ClientInfoPtr getClient(const std::string& clientId)
     {
-        for (auto client : clients)
+        for (const auto& client : clients)
         {
             if (client->id == clientId)
                 return client;
@@ -335,7 +336,7 @@ struct Group
         if (!client)
             return;
 
-        for (auto c : clients)
+        for (const auto& c : clients)
         {
             if (c->id == client->id)
                 return;
@@ -394,9 +395,13 @@ public:
 
     std::vector<GroupPtr> groups;
 
+    std::mutex& getMutex();
+
 private:
     Config() = default;
     ~Config();
+    mutable std::recursive_mutex mutex_;
+    std::mutex client_mutex_;
     std::string filename_;
 };
 
