@@ -53,7 +53,7 @@ void ControlServer::cleanup()
     auto count = distance(new_end, sessions_.end());
     if (count > 0)
     {
-        LOG(ERROR, LOG_TAG) << "Removing " << count << " inactive session(s), active sessions: " << sessions_.size() - count << "\n";
+        LOG(INFO, LOG_TAG) << "Removing " << count << " inactive session(s), active sessions: " << sessions_.size() - count << "\n";
         sessions_.erase(new_end, sessions_.end());
     }
 }
@@ -74,28 +74,27 @@ void ControlServer::send(const std::string& message, const ControlSession* exclu
 }
 
 
-std::string ControlServer::onMessageReceived(ControlSession* session, const std::string& message)
+void ControlServer::onMessageReceived(std::shared_ptr<ControlSession> session, const std::string& message, const ResponseHander& response_handler)
 {
     // LOG(DEBUG, LOG_TAG) << "received: \"" << message << "\"\n";
     if (controlMessageReceiver_ != nullptr)
-        return controlMessageReceiver_->onMessageReceived(session, message);
-    return "";
+        controlMessageReceiver_->onMessageReceived(std::move(session), message, response_handler);
 }
 
 
-void ControlServer::onNewSession(const shared_ptr<ControlSession>& session)
+void ControlServer::onNewSession(shared_ptr<ControlSession> session)
 {
     std::lock_guard<std::recursive_mutex> mlock(session_mutex_);
     session->start();
-    sessions_.emplace_back(session);
+    sessions_.emplace_back(std::move(session));
     cleanup();
 }
 
 
-void ControlServer::onNewSession(const std::shared_ptr<StreamSession>& session)
+void ControlServer::onNewSession(std::shared_ptr<StreamSession> session)
 {
     if (controlMessageReceiver_ != nullptr)
-        controlMessageReceiver_->onNewSession(session);
+        controlMessageReceiver_->onNewSession(std::move(session));
 }
 
 
@@ -135,8 +134,8 @@ void ControlServer::handleAccept(tcp::socket socket, Args&&... args)
         setsockopt(socket.native_handle(), SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));
         //	socket->set_option(boost::asio::ip::tcp::no_delay(false));
         LOG(NOTICE, LOG_TAG) << "ControlServer::NewConnection: " << socket.remote_endpoint().address().to_string() << endl;
-        shared_ptr<SessionType> session = make_shared<SessionType>(this, io_context_, std::move(socket), std::forward<Args>(args)...);
-        onNewSession(session);
+        shared_ptr<SessionType> session = make_shared<SessionType>(this, std::move(socket), std::forward<Args>(args)...);
+        onNewSession(std::move(session));
     }
     catch (const std::exception& e)
     {
@@ -156,8 +155,8 @@ void ControlServer::start()
             try
             {
                 LOG(INFO, LOG_TAG) << "Creating TCP acceptor for address: " << address << ", port: " << tcp_settings_.port << "\n";
-                acceptor_tcp_.emplace_back(
-                    make_unique<tcp::acceptor>(io_context_, tcp::endpoint(boost::asio::ip::address::from_string(address), tcp_settings_.port)));
+                acceptor_tcp_.emplace_back(make_unique<tcp::acceptor>(net::make_strand(io_context_.get_executor()),
+                                                                      tcp::endpoint(boost::asio::ip::address::from_string(address), tcp_settings_.port)));
             }
             catch (const boost::system::system_error& e)
             {
@@ -172,8 +171,8 @@ void ControlServer::start()
             try
             {
                 LOG(INFO, LOG_TAG) << "Creating HTTP acceptor for address: " << address << ", port: " << http_settings_.port << "\n";
-                acceptor_http_.emplace_back(
-                    make_unique<tcp::acceptor>(io_context_, tcp::endpoint(boost::asio::ip::address::from_string(address), http_settings_.port)));
+                acceptor_http_.emplace_back(make_unique<tcp::acceptor>(net::make_strand(io_context_.get_executor()),
+                                                                       tcp::endpoint(boost::asio::ip::address::from_string(address), http_settings_.port)));
             }
             catch (const boost::system::system_error& e)
             {
