@@ -19,13 +19,7 @@
 #ifndef SERVER_HPP
 #define SERVER_HPP
 
-#include <boost/asio.hpp>
-#include <memory>
-#include <mutex>
-#include <set>
-#include <sstream>
-#include <vector>
-
+// local headers
 #include "common/queue.h"
 #include "common/sample_format.hpp"
 #include "control_server.hpp"
@@ -38,6 +32,17 @@
 #include "stream_session.hpp"
 #include "streamreader/stream_manager.hpp"
 
+// 3rd party headers
+#include <boost/asio.hpp>
+
+// standard headers
+#include <memory>
+#include <mutex>
+#include <set>
+#include <sstream>
+#include <vector>
+
+
 using namespace streamreader;
 
 using boost::asio::ip::tcp;
@@ -45,16 +50,16 @@ using acceptor_ptr = std::unique_ptr<tcp::acceptor>;
 using session_ptr = std::shared_ptr<StreamSession>;
 
 
-/// Forwars PCM data to the connected clients
 /**
- * Reads PCM data using PipeStream, implements PcmListener to get the (encoded) PCM stream.
- * Accepts and holds client connections (StreamSession)
- * Receives (via the StreamMessageReceiver interface) and answers messages from the clients
- * Forwards PCM data to the clients
+ * Receives and routes PcmStreams
+ * Processes ControlSession messages
  */
-class Server : public StreamMessageReceiver, public ControlMessageReceiver, public PcmListener
+class Server : public StreamMessageReceiver, public ControlMessageReceiver, public PcmStream::Listener
 {
 public:
+    // TODO: revise handler names
+    using OnResponse = std::function<void(jsonrpcpp::entity_ptr response, jsonrpcpp::notification_ptr notification)>;
+
     Server(boost::asio::io_context& io_context, const ServerSettings& serverSettings);
     virtual ~Server();
 
@@ -67,28 +72,28 @@ private:
     void onDisconnect(StreamSession* streamSession) override;
 
     /// Implementation of ControllMessageReceiver
-    std::string onMessageReceived(ControlSession* controlSession, const std::string& message) override;
-    void onNewSession(const std::shared_ptr<ControlSession>& session) override
+    void onMessageReceived(std::shared_ptr<ControlSession> controlSession, const std::string& message, const ResponseHander& response_handler) override;
+    void onNewSession(std::shared_ptr<ControlSession> session) override
     {
         std::ignore = session;
     };
-    void onNewSession(const std::shared_ptr<StreamSession>& session) override;
+    void onNewSession(std::shared_ptr<StreamSession> session) override;
 
-    /// Implementation of PcmListener
-    void onMetaChanged(const PcmStream* pcmStream) override;
+    /// Implementation of PcmStream::Listener
+    void onPropertiesChanged(const PcmStream* pcmStream, const Properties& properties) override;
     void onStateChanged(const PcmStream* pcmStream, ReaderState state) override;
     void onChunkRead(const PcmStream* pcmStream, const msg::PcmChunk& chunk) override;
     void onChunkEncoded(const PcmStream* pcmStream, std::shared_ptr<msg::PcmChunk> chunk, double duration) override;
     void onResync(const PcmStream* pcmStream, double ms) override;
 
 private:
-    void processRequest(const jsonrpcpp::request_ptr request, jsonrpcpp::entity_ptr& response, jsonrpcpp::notification_ptr& notification) const;
+    void processRequest(const jsonrpcpp::request_ptr request, const OnResponse& on_response) const;
     /// Save the server state deferred to prevent blocking and lower disk io
     /// @param deferred the delay after the last call to saveConfig
     void saveConfig(const std::chrono::milliseconds& deferred = std::chrono::seconds(2));
 
-    mutable std::recursive_mutex sessionsMutex_;
-    mutable std::recursive_mutex clientMutex_;
+    // mutable std::recursive_mutex controlMutex_;
+    // mutable std::recursive_mutex clientMutex_;
     boost::asio::io_context& io_context_;
     boost::asio::steady_timer config_timer_;
 
