@@ -16,14 +16,20 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
+// prototype/interface header file
 #include "stream_server.hpp"
+
+// local headers
 #include "common/aixlog.hpp"
 #include "config.hpp"
 #include "message/client_info.hpp"
 #include "message/hello.hpp"
-#include "message/stream_tags.hpp"
 #include "message/time.hpp"
 #include "stream_session_tcp.hpp"
+
+// 3rd party headers
+
+// standard headers
 #include <iostream>
 
 using namespace std;
@@ -33,7 +39,7 @@ using json = nlohmann::json;
 
 static constexpr auto LOG_TAG = "StreamServer";
 
-StreamServer::StreamServer(boost::asio::io_context& io_context, const ServerSettings& serverSettings, StreamMessageReceiver* messageReceiver)
+StreamServer::StreamServer(net::io_context& io_context, const ServerSettings& serverSettings, StreamMessageReceiver* messageReceiver)
     : io_context_(io_context), config_timer_(io_context), settings_(serverSettings), messageReceiver_(messageReceiver)
 {
 }
@@ -48,40 +54,24 @@ void StreamServer::cleanup()
     auto count = distance(new_end, sessions_.end());
     if (count > 0)
     {
-        LOG(ERROR, LOG_TAG) << "Removing " << count << " inactive session(s), active sessions: " << sessions_.size() - count << "\n";
+        LOG(INFO, LOG_TAG) << "Removing " << count << " inactive session(s), active sessions: " << sessions_.size() - count << "\n";
         sessions_.erase(new_end, sessions_.end());
     }
 }
 
 
-void StreamServer::addSession(const std::shared_ptr<StreamSession>& session)
+void StreamServer::addSession(std::shared_ptr<StreamSession> session)
 {
     session->setMessageReceiver(this);
     session->setBufferMs(settings_.stream.bufferMs);
     session->start();
 
     std::lock_guard<std::recursive_mutex> mlock(sessionsMutex_);
-    sessions_.emplace_back(session);
+    sessions_.emplace_back(std::move(session));
     cleanup();
 }
 
 
-void StreamServer::onMetaChanged(const PcmStream* pcmStream, std::shared_ptr<msg::StreamTags> meta)
-{
-    // Send meta to all connected clients
-
-    std::lock_guard<std::recursive_mutex> mlock(sessionsMutex_);
-    for (const auto& s : sessions_)
-    {
-        if (auto session = s.lock())
-        {
-            if (session->pcmStream().get() == pcmStream)
-                session->send(meta);
-        }
-    }
-}
-
-
 void StreamServer::onChunkEncoded(const PcmStream* pcmStream, bool isDefaultStream, std::shared_ptr<msg::PcmChunk> chunk, double /*duration*/)
 {
     // LOG(TRACE, LOG_TAG) << "onChunkRead (" << pcmStream->getName() << "): " << duration << "ms\n";
@@ -100,6 +90,7 @@ void StreamServer::onChunkEncoded(const PcmStream* pcmStream, bool isDefaultStre
     {
         if (!settings_.stream.sendAudioToMutedClients)
         {
+            std::lock_guard<std::mutex> lock(Config::instance().getMutex());
             GroupPtr group = Config::instance().getGroupFromClient(session->clientId);
             if (group)
             {
@@ -109,7 +100,6 @@ void StreamServer::onChunkEncoded(const PcmStream* pcmStream, bool isDefaultStre
                 }
                 else
                 {
-                    std::lock_guard<std::recursive_mutex> lock(clientMutex_);
                     ClientInfoPtr client = group->getClient(session->clientId);
                     if (client && client->config.volume.muted)
                         continue;
@@ -211,7 +201,7 @@ void StreamServer::handleAccept(tcp::socket socket)
         socket.set_option(tcp::no_delay(true));
 
         LOG(NOTICE, LOG_TAG) << "StreamServer::NewConnection: " << socket.remote_endpoint().address().to_string() << endl;
-        shared_ptr<StreamSession> session = make_shared<StreamSessionTcp>(io_context_, this, std::move(socket));
+        shared_ptr<StreamSession> session = make_shared<StreamSessionTcp>(this, std::move(socket));
         addSession(session);
     }
     catch (const std::exception& e)
@@ -229,8 +219,8 @@ void StreamServer::start()
         try
         {
             LOG(INFO, LOG_TAG) << "Creating stream acceptor for address: " << address << ", port: " << settings_.stream.port << "\n";
-            acceptor_.emplace_back(
-                make_unique<tcp::acceptor>(io_context_, tcp::endpoint(boost::asio::ip::address::from_string(address), settings_.stream.port)));
+            acceptor_.emplace_back(make_unique<tcp::acceptor>(net::make_strand(io_context_.get_executor()),
+                                                              tcp::endpoint(boost::asio::ip::address::from_string(address), settings_.stream.port)));
         }
         catch (const boost::system::system_error& e)
         {
