@@ -19,13 +19,7 @@
 #ifndef STREAM_SERVER_HPP
 #define STREAM_SERVER_HPP
 
-#include <boost/asio.hpp>
-#include <memory>
-#include <mutex>
-#include <set>
-#include <sstream>
-#include <vector>
-
+// local headers
 #include "common/queue.h"
 #include "common/sample_format.hpp"
 #include "control_server.hpp"
@@ -37,6 +31,17 @@
 #include "stream_session.hpp"
 #include "streamreader/stream_manager.hpp"
 
+// 3rd party headers
+#include <boost/asio.hpp>
+
+// standard headers
+#include <memory>
+#include <mutex>
+#include <set>
+#include <sstream>
+#include <vector>
+
+
 using namespace streamreader;
 
 using boost::asio::ip::tcp;
@@ -46,7 +51,7 @@ using session_ptr = std::shared_ptr<StreamSession>;
 
 /// Forwars PCM data to the connected clients
 /**
- * Reads PCM data using PipeStream, implements PcmListener to get the (encoded) PCM stream.
+ * Reads PCM data from several StreamSessions
  * Accepts and holds client connections (StreamSession)
  * Receives (via the StreamMessageReceiver interface) and answers messages from the clients
  * Forwards PCM data to the clients
@@ -54,7 +59,7 @@ using session_ptr = std::shared_ptr<StreamSession>;
 class StreamServer : public StreamMessageReceiver
 {
 public:
-    StreamServer(boost::asio::io_context& io_context, const ServerSettings& serverSettings, StreamMessageReceiver* messageReceiver = nullptr);
+    StreamServer(net::io_context& io_context, const ServerSettings& serverSettings, StreamMessageReceiver* messageReceiver = nullptr);
     virtual ~StreamServer();
 
     void start();
@@ -63,8 +68,8 @@ public:
     /// Send a message to all connceted clients
     //	void send(const msg::BaseMessage* message);
 
-    void addSession(const std::shared_ptr<StreamSession>& session);
-    void onMetaChanged(const PcmStream* pcmStream, std::shared_ptr<msg::StreamTags> meta);
+    void addSession(std::shared_ptr<StreamSession> session);
+    // void onMetadataChanged(const PcmStream* pcmStream, std::shared_ptr<msg::StreamTags> meta);
     void onChunkEncoded(const PcmStream* pcmStream, bool isDefaultStream, std::shared_ptr<msg::PcmChunk> chunk, double duration);
 
     session_ptr getStreamSession(const std::string& clientId) const;
@@ -80,9 +85,9 @@ private:
     void onDisconnect(StreamSession* streamSession) override;
 
     mutable std::recursive_mutex sessionsMutex_;
-    mutable std::recursive_mutex clientMutex_;
+    // mutable std::recursive_mutex clientMutex_;
     std::vector<std::weak_ptr<StreamSession>> sessions_;
-    boost::asio::io_context& io_context_;
+    net::io_context& io_context_;
     std::vector<acceptor_ptr> acceptor_;
     boost::asio::steady_timer config_timer_;
 
