@@ -19,8 +19,11 @@
 #ifndef META_STREAM_HPP
 #define META_STREAM_HPP
 
+// local headers
 #include "posix_stream.hpp"
 #include "resampler.hpp"
+
+// standard headers
 #include <memory>
 
 namespace streamreader
@@ -31,21 +34,38 @@ namespace streamreader
 /**
  * Reads PCM and passes the data to an encoder.
  * Implements EncoderListener to get the encoded data.
- * Data is passed to the PcmListener
+ * Data is passed to the PcmStream::Listener
  */
-class MetaStream : public PcmStream, public PcmListener
+class MetaStream : public PcmStream, public PcmStream::Listener
 {
 public:
-    /// ctor. Encoded PCM data is passed to the PcmListener
-    MetaStream(PcmListener* pcmListener, const std::vector<std::shared_ptr<PcmStream>>& streams, boost::asio::io_context& ioc, const StreamUri& uri);
+    /// ctor. Encoded PCM data is passed to the PcmStream::Listener
+    MetaStream(PcmStream::Listener* pcmListener, const std::vector<std::shared_ptr<PcmStream>>& streams, boost::asio::io_context& ioc,
+               const ServerSettings& server_settings, const StreamUri& uri);
     virtual ~MetaStream();
 
     void start() override;
     void stop() override;
 
+    // Setter for properties
+    void setShuffle(bool shuffle, ResultHandler handler) override;
+    void setLoopStatus(LoopStatus status, ResultHandler handler) override;
+    void setVolume(uint16_t volume, ResultHandler handler) override;
+    void setRate(float rate, ResultHandler handler) override;
+
+    // Control commands
+    void setPosition(std::chrono::milliseconds position, ResultHandler handler) override;
+    void seek(std::chrono::milliseconds offset, ResultHandler handler) override;
+    void next(ResultHandler handler) override;
+    void previous(ResultHandler handler) override;
+    void pause(ResultHandler handler) override;
+    void playPause(ResultHandler handler) override;
+    void stop(ResultHandler handler) override;
+    void play(ResultHandler handler) override;
+
 protected:
-    /// Implementation of PcmListener
-    void onMetaChanged(const PcmStream* pcmStream) override;
+    /// Implementation of PcmStream::Listener
+    void onPropertiesChanged(const PcmStream* pcmStream, const Properties& properties) override;
     void onStateChanged(const PcmStream* pcmStream, ReaderState state) override;
     void onChunkRead(const PcmStream* pcmStream, const msg::PcmChunk& chunk) override;
     void onChunkEncoded(const PcmStream* pcmStream, std::shared_ptr<msg::PcmChunk> chunk, double duration) override;
@@ -53,8 +73,8 @@ protected:
 
 protected:
     std::vector<std::shared_ptr<PcmStream>> streams_;
+    std::recursive_mutex active_mutex_;
     std::shared_ptr<PcmStream> active_stream_;
-    std::mutex mutex_;
     std::unique_ptr<Resampler> resampler_;
     bool first_read_;
     std::chrono::time_point<std::chrono::steady_clock> next_tick_;
